@@ -1,9 +1,7 @@
 /*
- * bindings.h                 Bindings
+ * bmon/bindings.h		Bindings
  *
- * $Id: bindings.h 1 2004-10-17 17:32:34Z tgr $
- *
- * Copyright (c) 2001-2004 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2001-2005 Thomas Graf <tgraf@suug.ch>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,17 +26,17 @@
 #define __BMON_BINDINGS_H_
 
 #include <bmon/bmon.h>
-#include <bmon/intf.h>
+#include <bmon/item.h>
 
 typedef struct binding_s
 {
-    int ch;
-    char *cmd;
-    char *args[256];
-    struct binding_s *next;
+	int ch;
+	char *cmd;
+	char *args[256];
+	struct binding_s *next;
 } binding_t;
 
-extern void add_binding(binding_t *b);
-extern int handle_bindings(int ch, const char *args);
+extern void add_binding(binding_t *);
+extern int handle_bindings(int, const char *);
 
 #endif
